
open Format
open Base
open Ast

let string_of_op = function
  | Plus -> "+"
  | Mult -> "*"
  | Div -> "/"
  | Minus -> "-"
  | Eq -> "="
  | Leq -> "<="
  | Geq -> ">="
  | Lt -> "<"
  | Gt -> ">"

let string_of_type = function
  | TypInt -> "int"
  | TypBool -> "bool"

let rec pp_concat pp f = function
  | [] -> ()
  | [e] -> pp f e
  | e::tl -> fprintf f "%a,%a" pp e (pp_concat pp) tl

let rec print_expr paren f e = match e.asa with
  | Cst (Int i) -> fprintf f "%i" i
  | Cst (Bool b) -> fprintf f "%b" b
  | Var s -> fprintf f "%s" s
  | Call(id,l) ->
      fprintf f
        "%s(@[%a@])"
        id
        (pp_concat (print_expr false)) l
  | asa ->
      (if paren then fprintf f "(%a)" else (fun pp x -> pp f x))
        (fun f -> function
           | Binop (o,e1,e2) ->
               fprintf f
                 "@[<2>%a@]%s@[<2>%a@]"
                 (print_expr true) e1
                 (string_of_op o)
                 (print_expr true) e2
           | If (c,t,e) ->
               fprintf f
                 "@[<2>if@ @[<2>%a@] then@ @[<2>%a@]@ else@ @[<2>%a@]@]"
                 (print_expr false) c
                 (print_expr false) t
                 (print_expr false) e
           | Letin (id,d,b) ->
               fprintf f
                 "let %s = %a in %a"
                 id
                 (print_expr false) d
                 (print_expr false) b
           | _ -> assert false)
        asa

let rec print_args = pp_concat (fun f (x,t) -> fprintf f "%s:%s" x (string_of_type t))

let print_decl f d = match d.asa with
  | Vdecl (s,e) ->
      fprintf f
        "@[<2>let %s =@ %a@]@."
        s
        (print_expr false) e
  | Fdecl (id,args,t,e) ->
      fprintf f
        "@[<2>let %s(%a):%s =@ %a@]@."
        id
        print_args args
        (string_of_type t)
        (print_expr false) e

let print_prog f = List.iter (print_decl f)
